/***********      .---.         .-"-.      *******************\
* -------- *     /   ._.       /  ` \     * ---------------- *
* Author's *     \_  (__\      \_v_/     * humus@rogers.com *
*   note   *     //   \\       //   \\     * ICQ #47010716    *
* -------- *    ((     ))     ((     ))    * ---------------- *
*          ****--""---""-------""---""--****                  ********\
* This file is a part of the work done by Humus. You are free to use  *
* the code in any way you like, modified, unmodified or copy'n'pasted *
* into your own work. However, I expect you to respect these points:  *
*  @ If you use this file and its contents unmodified, or use a major *
*    part of this file, please credit the author and leave this note. *
*  @ For use in anything commercial, please request my approval.      *
*  @ Share your work and ideas too as much as you can.                *
\*********************************************************************/

#ifndef _AUDIO_H_
#define _AUDIO_H_

#include "../Util/Set.h"
#include "AudioStream.h"

#include <AL/al.h>
#include <AL/alc.h>



typedef int SoundID;

#define SOUND_NONE    (-1)
#define SOUND_UNDEFINED ((unsigned int) (-2))

// Sound flags
#define SND_LOOPING    0x1
#define SND_STREAM     0x2


struct Sound {
	AudioStream *audioStream;
	unsigned char *samples[2];
	
	ALenum format;
	ALuint source;
	ALuint buffer[2];

	unsigned int currStream;
	bool endOfStream;
	bool isStreaming;
	bool isLooping;
};

class Audio {
protected:
	ALCcontext *ctx;
//	void *ctx;
	ALCdevice *dev;

	Set <Sound> sounds;

	SoundID insertSound(Sound &sound);
	void removeSound(Sound &sound);
	void deleteSound(const int index);
public:
	Audio();
	~Audio();

	void clear();

	SoundID addSound(const char *fileName, unsigned int flags = 0);
	bool playSound(const SoundID soundID);
	bool stopSound(const SoundID soundID);
	bool pauseSound(const SoundID soundID);

	bool updateSoundStream(const SoundID soundID);
};

#endif // _AUDIO_H_
