/***********      .---.         .-"-.      *******************\
* -------- *     /   ._.       /  ` \     * ---------------- *
* Author's *     \_  (__\      \_v_/     * humus@rogers.com *
*   note   *     //   \\       //   \\     * ICQ #47010716    *
* -------- *    ((     ))     ((     ))    * ---------------- *
*          ****--""---""-------""---""--****                  ********\
* This file is a part of the work done by Humus. You are free to use  *
* the code in any way you like, modified, unmodified or copy'n'pasted *
* into your own work. However, I expect you to respect these points:  *
*  @ If you use this file and its contents unmodified, or use a major *
*    part of this file, please credit the author and leave this note. *
*  @ For use in anything commercial, please request my approval.      *
*  @ Share your work and ideas too as much as you can.                *
\*********************************************************************/

#ifndef _AUDIOSTREAM_H_
#define _AUDIOSTREAM_H_

#include <stdio.h>
#include "../Util/Fifo.h"
#include "../Platform.h"

#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>


#define WHOLE_FILE 0xFFFFFFFF

enum SNDTYPE {
	WAV,
	OGG,
};


class AudioStream {
private:
	Fifo <unsigned char *> data;
	
	unsigned int bytesPerSample;
	unsigned int nChannels;
	unsigned int frequency;

	unsigned int blockSize;
	unsigned int currPos;
	unsigned int bufferedSize;
	unsigned int currWritePos;

	OggVorbis_File vf;
	SNDTYPE sndType;
	FILE *file;
	char *name;


public:
	AudioStream();
	~AudioStream();

	unsigned int getBytesPerSample() const { return bytesPerSample; }
	unsigned int getTotalSampleBytes() const { return bytesPerSample * nChannels; }
	unsigned int getChannels() const { return nChannels; }
	unsigned int getFrequency() const { return frequency; }

	char *getName() const { return name; }

	bool open(const char *fileName);
	unsigned int read(unsigned char **buffer, unsigned int numSamples = WHOLE_FILE);
	void close();

	bool restart();
};



#endif // _AUDIOSTREAM_H_
