/***********      .---.         .-"-.      *******************\
* -------- *     /   ._.       /  ` \     * ---------------- *
* Author's *     \_  (__\      \_v_/     * humus@rogers.com *
*   note   *     //   \\       //   \\     * ICQ #47010716    *
* -------- *    ((     ))     ((     ))    * ---------------- *
*          ****--""---""-------""---""--****                  ********\
* This file is a part of the work done by Humus. You are free to use  *
* the code in any way you like, modified, unmodified or copy'n'pasted *
* into your own work. However, I expect you to respect these points:  *
*  @ If you use this file and its contents unmodified, or use a major *
*    part of this file, please credit the author and leave this note. *
*  @ For use in anything commercial, please request my approval.      *
*  @ Share your work and ideas too as much as you can.                *
\*********************************************************************/

#include "OpenGLRenderTarget.h"

#ifdef WIN32

bool OpenGLRenderTarget::bind(){

	return (wglBindTexImageARB(hPBuffer, WGL_FRONT_LEFT_ARB) == GL_TRUE);
}

bool OpenGLRenderTarget::unbind(){
	return (wglReleaseTexImageARB(hPBuffer, WGL_FRONT_LEFT_ARB) == GL_TRUE);
}

bool OpenGLRenderTarget::create(const int w, const int h, const bool mipmapped, const bool cubemap, RenderPixelFormat rPixelFormat){
	width  = w;
	height = h;

	int nBitsPerChannel = getChannelBits(rPixelFormat);

	HDC hdc = wglGetCurrentDC();
	HGLRC hglrc = wglGetCurrentContext();

    int pixelFormat;
	unsigned int nMatchingPixelFormats;

	int iAttributeIList[] = {
		WGL_SUPPORT_OPENGL_ARB,       GL_TRUE, 
		

		WGL_DRAW_TO_PBUFFER_ARB,      GL_TRUE,
		WGL_BIND_TO_TEXTURE_RGBA_ARB, GL_TRUE,


		WGL_PIXEL_TYPE_ARB,           isFloatPixelFormat(rPixelFormat)? WGL_TYPE_RGBA_FLOAT_ATI : WGL_TYPE_RGBA_ARB,
		WGL_ACCELERATION_ARB,         WGL_FULL_ACCELERATION_ARB,
		
		WGL_RED_BITS_ARB,             nBitsPerChannel,
		WGL_GREEN_BITS_ARB,           nBitsPerChannel,
		WGL_BLUE_BITS_ARB,            nBitsPerChannel,
		WGL_ALPHA_BITS_ARB,           nBitsPerChannel,

		WGL_DEPTH_BITS_ARB,           24,
		WGL_STENCIL_BITS_ARB,         0,
		WGL_DOUBLE_BUFFER_ARB,        GL_FALSE,

		0, 0 // End of array
	};

	wglChoosePixelFormatARB(hdc, iAttributeIList, NULL, 1, &pixelFormat, &nMatchingPixelFormats);

/*	if (nMatchingPixelFormats <= 0){
		::MessageBox(NULL, "No matching format", "Error", MB_OK);
	}*/

	
	int iPBufferAttributes[] = {
		WGL_PBUFFER_LARGEST_ARB,   GL_TRUE,
		WGL_TEXTURE_FORMAT_ARB,    WGL_TEXTURE_RGBA_ARB,
		WGL_TEXTURE_TARGET_ARB,    cubemap? WGL_TEXTURE_CUBE_MAP_ARB : WGL_TEXTURE_2D_ARB,
		WGL_MIPMAP_TEXTURE_ARB,    mipmapped? GL_TRUE : GL_FALSE,
		0, 0,
	};


	glGenTextures(1, &texID);
	glBindTexture(cubemap? GL_TEXTURE_CUBE_MAP_ARB : GL_TEXTURE_2D, texID);

	hPBuffer = wglCreatePbufferARB(hdc, pixelFormat, width, height, iPBufferAttributes);

	hPdc = wglGetPbufferDCARB(hPBuffer);
	hPrc = wglCreateContext(hPdc);

	wglShareLists(hglrc, hPrc);

	GLenum target = cubemap? GL_TEXTURE_CUBE_MAP_ARB : GL_TEXTURE_2D;
	glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE_EXT);
	glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE_EXT);
	glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(target, GL_TEXTURE_MIN_FILTER, mipmapped? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR);
	if (mipmapped) glTexParameteri(target, GL_GENERATE_MIPMAP_SGIS, GL_TRUE);

//	glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, 0, 0, width, height, 0);



	return true;
}

bool OpenGLRenderTarget::destroy(){
	wglDeleteContext(hPrc);
	wglReleasePbufferDCARB(hPBuffer, hPdc);
	wglDestroyPbufferARB(hPBuffer);
	return true;
}

bool OpenGLRenderTarget::setCurrent(){
    wglMakeCurrent(hPdc, hPrc);
	return true;
}

bool OpenGLRenderTarget::unsetCurrent(){
	//glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, 0, 192, width, height, 0);
	//glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 192, width, 64);
	return true;
}

bool OpenGLRenderTarget::isLost(){
	int lost = 1;

	wglQueryPbufferARB(hPBuffer, WGL_PBUFFER_LOST_ARB, &lost);
	
	return (lost != 0);
}

bool OpenGLRenderTarget::setRenderFace(const int face){
	glFlush();

	int iPBufferAttributes[] = {
		WGL_CUBE_MAP_FACE_ARB, WGL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB + face,
		0, 0
	};

	return (wglSetPbufferAttribARB(hPBuffer, iPBufferAttributes) == TRUE);
}

#endif // WIN32







#ifdef LINUX

#include <stdio.h>

bool OpenGLRenderTarget::bind(){
	glXBindTexImageATI(display, pBuffer, GLX_FRONT_LEFT_ATI);
	return true;
}

bool OpenGLRenderTarget::unbind(){
	glXReleaseTexImageATI(display, pBuffer, GLX_FRONT_LEFT_ATI);
	return true;
}

bool OpenGLRenderTarget::create(const int w, const int h, const bool mipmapped, const bool cubemap, RenderPixelFormat rPixelFormat){
	width  = w;
	height = h;

	int nBitsPerChannel = getChannelBits(rPixelFormat);
	
	int attList[] = {
		GLX_BIND_TO_TEXTURE_RGBA_ATI, True,
//		GLX_RENDER_TYPE,   isFloatPixelFormat(rPixelFormat)? GLX_RGBA_FLOAT_ATI_BIT : GLX_RGBA_BIT,
		GLX_X_RENDERABLE,  True,
		GLX_DRAWABLE_TYPE, GLX_PBUFFER_BIT,
		GLX_X_VISUAL_TYPE, GLX_TRUE_COLOR,
		GLX_CONFIG_CAVEAT, GLX_NONE,
		GLX_RED_SIZE,      nBitsPerChannel,
		GLX_GREEN_SIZE,    nBitsPerChannel,
		GLX_BLUE_SIZE,     nBitsPerChannel,
		GLX_ALPHA_SIZE,    nBitsPerChannel,
		GLX_DEPTH_SIZE,    24,
		GLX_STENCIL_SIZE,  0,
		GLX_DOUBLEBUFFER,  False,
		None
	};

	int nConfigs = 0;
	GLXFBConfig *configs = glXChooseFBConfig(display, screen, attList, &nConfigs);
	if (nConfigs == 0 || !configs){
		printf("No matching FBConfig\n");
		return false;
	}

	int bestConfig = 0;
	int minBits = 0x7FFFFFFF;
	for (int i = 0; i < nConfigs; i++){
		int red, green, blue, alpha, depth, stencil;
		glXGetFBConfigAttrib(display, configs[i], GLX_RED_SIZE,     &red);
		glXGetFBConfigAttrib(display, configs[i], GLX_GREEN_SIZE,   &green);
		glXGetFBConfigAttrib(display, configs[i], GLX_BLUE_SIZE,    &blue);
		glXGetFBConfigAttrib(display, configs[i], GLX_ALPHA_SIZE,   &alpha);
		glXGetFBConfigAttrib(display, configs[i], GLX_DEPTH_SIZE,   &depth);
		glXGetFBConfigAttrib(display, configs[i], GLX_STENCIL_SIZE, &stencil);

		int bits = red + green + blue + alpha + depth + stencil;
		if (bits < minBits){
			bestConfig = i;
			minBits = bits;
		}
	}

	GLXFBConfig config = configs[bestConfig];
	XFree(configs);
	
	int pBufAtt[] = {
		GLX_PBUFFER_WIDTH,  width,
		GLX_PBUFFER_HEIGHT, height,

		GLX_LARGEST_PBUFFER, True,
		GLX_TEXTURE_FORMAT_ATI, GLX_TEXTURE_RGBA_ATI,
		GLX_TEXTURE_TARGET_ATI, cubemap? GLX_TEXTURE_CUBE_MAP_ATI : GLX_TEXTURE_2D_ATI,
		GLX_MIPMAP_TEXTURE_ATI, mipmapped? True : False,
		None
	};

	pBuffer = glXCreatePbuffer(display, config, pBufAtt);
	ctx = glXCreateNewContext(display, config, GLX_RGBA_TYPE, mainCtx, GL_TRUE);
	if (ctx == NULL){
		printf("Couldn't create context\n");
		return false;
	}
	
	glGenTextures(1, &texID);
	glBindTexture(cubemap? GL_TEXTURE_CUBE_MAP_ARB : GL_TEXTURE_2D, texID);

	GLenum target = cubemap? GL_TEXTURE_CUBE_MAP_ARB : GL_TEXTURE_2D;
	glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE_EXT);
	glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE_EXT);
	glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(target, GL_TEXTURE_MIN_FILTER, mipmapped? GL_LINEAR_MIPMAP_LINEAR : GL_LINEAR);
	if (mipmapped) glTexParameteri(target, GL_GENERATE_MIPMAP_SGIS, GL_TRUE);

	return true;
}

bool OpenGLRenderTarget::destroy(){
	glXDestroyContext(display, ctx);
	glXDestroyPbuffer(display, pBuffer);
	return true;
}

bool OpenGLRenderTarget::setCurrent(){
	glXMakeCurrent(display, pBuffer, ctx);
	return true;
}

bool OpenGLRenderTarget::unsetCurrent(){
	glXMakeCurrent(display, window, mainCtx);
	return true;
}

bool OpenGLRenderTarget::isLost(){
	return false;
}

bool OpenGLRenderTarget::setRenderFace(const int face){
	glFlush();

	int pBufAtt[] = {
		GLX_CUBE_MAP_FACE_ATI, GLX_TEXTURE_CUBE_MAP_POSITIVE_X_ATI + face,
		None
	};

	glXDrawableAttribATI(display, pBuffer, pBufAtt);
	return true;
}

#endif // LINUX
