/***********      .---.         .-"-.      *******************\
* -------- *     /   ._.       /  ` \     * ---------------- *
* Author's *     \_  (__\      \_v_/     * humus@rogers.com *
*   note   *     //   \\       //   \\     * ICQ #47010716    *
* -------- *    ((     ))     ((     ))    * ---------------- *
*          ****--""---""-------""---""--****                  ********\
* This file is a part of the work done by Humus. You are free to use  *
* the code in any way you like, modified, unmodified or copy'n'pasted *
* into your own work. However, I expect you to respect these points:  *
*  @ If you use this file and its contents unmodified, or use a major *
*    part of this file, please credit the author and leave this note. *
*  @ For use in anything commercial, please request my approval.      *
*  @ Share your work and ideas too as much as you can.                *
\*********************************************************************/


#ifndef _OPENGLRENDERER_H_
#define _OPENGLRENDERER_H_

#include "../Renderer.h"
#include "glExtensions.h"

class OpenGLRenderer : public Renderer {
public:

#if defined(WIN32)
	HDC hdc;
	HGLRC hglrc;
	OpenGLRenderer(HDC hDc, HGLRC hGlrc);

#elif defined(LINUX)

    GLXContext glxc;
	Display *display;
	int screen;
	Window window;
	OpenGLRenderer(GLXContext glXc, Display *disp, int scr, Window win);
#endif

protected:
	void init();
	static GLenum targets[];

	void createTexture(Texture &texture);
	void removeTexture(Texture &texture);
	bool uploadTexture(Texture &texture);
	void resetTextureUnits();

	void createFragmentShader(FragmentShader &fragmentShader);
	void removeFragmentShader(FragmentShader &fragmentShader);
	void uploadFragmentShader(FragmentShader &fragmentShader);

	void createVertexShader(VertexShader &vertexShader);
	void removeVertexShader(VertexShader &vertexShader);
	void uploadVertexShader(VertexShader &vertexShader);

	void configureTexEnv(const TexEnv texEnv, bool isRGB);
public:

	bool changeToDefaultRenderTarget();

	void changeTextureUnit(const int textureUnit);
	void changeTexture(const TextureID textureID);
	void changeTexEnv(const TexEnv colorTexEnv, const TexEnv alphaTexEnv);
	void changeTexGen(const TexGen texGen);
	void changeFragmentShader(const ShaderID shader);
	void changeVertexShader(const ShaderID shader);

	void changeBlending(const int srcFactor, const int destFactor, const int blendMode);
	void changeVSync(const bool value);
	void changeAnisotropic(const TextureID textureID, const bool value);
};


#endif // _OPENGLRENDERER_H_
