/***********      .---.         .-"-.      *******************\
* -------- *     /   ._.       /  ` \     * ---------------- *
* Author's *     \_  (__\      \_v_/     * humus@rogers.com *
*   note   *     //   \\       //   \\     * ICQ #47010716    *
* -------- *    ((     ))     ((     ))    * ---------------- *
*          ****--""---""-------""---""--****                  ********\
* This file is a part of the work done by Humus. You are free to use  *
* the code in any way you like, modified, unmodified or copy'n'pasted *
* into your own work. However, I expect you to respect these points:  *
*  @ If you use this file and its contents unmodified, or use a major *
*    part of this file, please credit the author and leave this note. *
*  @ For use in anything commercial, please request my approval.      *
*  @ Share your work and ideas too as much as you can.                *
\*********************************************************************/

#ifndef _RENDERTARGET_H_
#define _RENDERTARGET_H_

enum RenderPixelFormat {
	RPF_RGBA8   = 0,
	RPF_R16     = 1,
	RPF_RG16    = 2,
	RPF_RGBA16  = 3,
	RPF_R16f    = 4,
	RPF_RG16f   = 5,
	RPF_RGBA16f = 6,
	RPF_R32f    = 7,
	RPF_RG32f   = 8,
	RPF_RGBA32f = 9,
};

inline int getChannelBits(const RenderPixelFormat rpf){
	switch(rpf){
	case RPF_RGBA8:
		return 8;
	case RPF_R16:
	case RPF_RG16:
	case RPF_RGBA16:
	case RPF_R16f:
	case RPF_RG16f:
	case RPF_RGBA16f:
		return 16;
	case RPF_R32f:
	case RPF_RG32f:
	case RPF_RGBA32f:
		return 32;
	default:
		return 0;
	}
}

inline bool isFloatPixelFormat(const RenderPixelFormat rpf){
	return (rpf >= RPF_R16f);
}

class RenderTarget {
protected:
	unsigned int width, height;
	unsigned int texID;
public:
	unsigned int getTexID() const { return texID; }

	virtual bool bind() = 0;
	virtual bool unbind() = 0;

	virtual bool create(const int w, const int h, const bool mipmapped, const bool cubemap, RenderPixelFormat rPixelFormat) = 0;
	virtual bool destroy() = 0;

	virtual bool setCurrent() = 0;
	virtual bool unsetCurrent() = 0;
	
	virtual bool isLost() = 0;
	virtual bool setRenderFace(const int face) = 0;
};

#endif // _RENDERTARGET_H_
