/***********      .---.         .-"-.      *******************\
* -------- *     /   ._.       /  ` \     * ---------------- *
* Author's *     \_  (__\      \_v_/     * humus@rogers.com *
*   note   *     //   \\       //   \\     * ICQ #47010716    *
* -------- *    ((     ))     ((     ))    * ---------------- *
*          ****--""---""-------""---""--****                  ********\
* This file is a part of the work done by Humus. You are free to use  *
* the code in any way you like, modified, unmodified or copy'n'pasted *
* into your own work. However, I expect you to respect these points:  *
*  @ If you use this file and its contents unmodified, or use a major *
*    part of this file, please credit the author and leave this note. *
*  @ For use in anything commercial, please request my approval.      *
*  @ Share your work and ideas too as much as you can.                *
\*********************************************************************/

#include "Light.h"

Light::Light(const Vertex &pos, const Color &col, const float Size){
	position = pos;
	color = col;
	size = Size;
	corona = false;
}

void Light::setCorona(const Vertex &CoronaDir, const float CoronaSizeFactor, const bool DoubleSided){
	coronaDir = CoronaDir;
	coronaDir.normalize();
	coronaSizeFactor = CoronaSizeFactor;
	corona = true;
	doubleSided = DoubleSided;
}

void Light::setLastTime(const bool visible, const float time){
	if (visible){
		lastVisibleTime = time;
	} else {
		lastHiddenTime = time;
	}
}

const float Light::getCoronaSize(const Vertex &pos, const float currTime){
	Vertex lightVector = pos - position;
	float factor, angular, len = length(lightVector);
	lightVector /= len;
	
	if (lastVisibleTime >= lastHiddenTime){
		factor = 0.00005f * (currTime - lastHiddenTime);
		if (factor > 1) factor = 1;
	} else {
		factor = 1 + 0.00005f * (lastVisibleTime - currTime);
		if (factor < 0) factor = 0;
	}
	
	angular = lightVector * coronaDir;
	if (doubleSided) angular = fabsf(angular);
	return factor * size * coronaSizeFactor * sinf(PI - PI * len / (len + 700.0f)) * angular;
}
