/***********      .---.         .-"-.      *******************\
* -------- *     /   ._.       /  ` \     * ---------------- *
* Author's *     \_  (__\      \_v_/     * humus@rogers.com *
*   note   *     //   \\       //   \\     * ICQ #47010716    *
* -------- *    ((     ))     ((     ))    * ---------------- *
*          ****--""---""-------""---""--****                  ********\
* This file is a part of the work done by Humus. You are free to use  *
* the code in any way you like, modified, unmodified or copy'n'pasted *
* into your own work. However, I expect you to respect these points:  *
*  @ If you use this file and its contents unmodified, or use a major *
*    part of this file, please credit the author and leave this note. *
*  @ For use in anything commercial, please request my approval.      *
*  @ Share your work and ideas too as much as you can.                *
\*********************************************************************/

#ifndef _RENDERWINDOW_H_
#define _RENDERWINDOW_H_

#include <afxwin.h>
#include <atlbase.h>
#include <afxcmn.h>
#include "../CPU.h"


#include "../Math/Vertex.h"
#include "../DisplayMode.h"

#define VK_ENTER 13

#ifndef NO_UI
#define ID_FILE_EXIT                     100
#define ID_FILE_RESTART                  101
#define ID_FILE_CONSOLE                  102
#define ID_FULLSCREEN                    200
#define ID_PREFERENCES_CONFIGURECONTROLS 300
#define ID_PREFERENCES_INVERTMOUSE       301
#define ID_PREFERENCES_VSYNC             302
#define ID_PREFERENCES_ANISOTROPIC       303
#define ID_PREFERENCES_SHOWDEBUGINFO     304
#define ID_PREFERENCES_FSAA_OFF          305
#define ID_PREFERENCES_FSAA_2X           306
#define ID_PREFERENCES_FSAA_4X           307
#define ID_PREFERENCES_FSAA_6X           308
#define ID_HELP_ABOUT                    400

#define IDC_COMMANDLINE  1000
#define IDC_OUTPUT       1001
#endif

enum CREATE_ACTION {
	TERMINATE,
	CREATE_DEFAULT,
	CREATE_FULLSCREEN,
	CREATE_WINDOWED
};

class RenderWindow;

#include "RenderApp.h"


class RenderWindow : public CFrameWnd {
public:
    RenderWindow(RenderApp *app, CREATE_ACTION createAction);

	void setMouseSpeed(const DWORD speed){ mouseSpeed = speed; }
	DWORD getMouseSpeed() const { return mouseSpeed; }
	bool getFullscreen() const { return fullScreen; }

	DisplayModeHandler *getModes() const { return modes; }
protected:
	RenderApp *renderApp;

	DisplayModeHandler *modes;
	UINT prevCheckedMode;

#ifndef NO_UI
	CRichEditCtrl commandLine, output;
#endif


	CRect rect;
	int width, height, bpp, refreshRate;
	int middleX, middleY;
	bool fullScreen;
	bool captureMouse;

	bool leftButton, rightButton;
	DWORD mouseSpeed;
	DWORD fsaaLevel;
	bool colorMipMaps;
	bool invertMouse;

	bool vSync, anisotropic, showDebugInfo;
	bool loaded;


	BOOL PreCreateWindow(CREATESTRUCT &cs);
	int  OnCreate(LPCREATESTRUCT lpCreateStruct);
	void OnSize(UINT nType, int cx, int cy);
	void OnWindowPosChanged(WINDOWPOS *lpwndpos);
	void OnClose();

	void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	void OnKeyUp  (UINT nChar, UINT nRepCnt, UINT nFlags);
	void OnSysKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);

	void OnLButtonDown(UINT nFlags, CPoint point);
	void OnLButtonUp  (UINT nFlags, CPoint point);
	void OnRButtonDown(UINT nFlags, CPoint point);
	void OnRButtonUp  (UINT nFlags, CPoint point);
	void OnMouseMove  (UINT nFlags, CPoint point);

	void OnKillFocus(CWnd *pNewWnd);

#ifndef NO_UI
	void OnCommandLineNotify(NMHDR *pNMHDR, LRESULT *pResult);

	void OnFileExit();
	void OnFileRestart();
	void OnFileConsole();
	void OnFullscreen();
	void OnFullscreenMode(UINT nID);
	void OnPreferencesConfigureControls();
	void OnPreferencesInvertMouse();
	void OnPreferencesVSync();
	void OnPreferencesAnisotropic();
	void OnPreferencesFsaa(UINT nID);
	void OnShowDebugInfo();
	void OnHelpAbout();
#endif

	DECLARE_MESSAGE_MAP ()
};

#endif // _RENDERWINDOW_H_
